import pytest


@pytest.fixture
def torr_test_file(datafix_dir):
    return str(datafix_dir / 'test_file.torrent')


@pytest.fixture
def torr_test_dir(datafix_dir):
    return str(datafix_dir / 'test_dir.torrent')


@pytest.fixture(scope='session')
def struct_torr_dir():
    return {
        'announce': 'http://track1.org/1/',
        'announce-list': [
            ['http://track1.org/1/', 'http://track2.org/2/']
        ],
        'comment': 'примечание',
        'created by': 'Transmission/2.84 (14307)',
        'creation date': 1445766124,
        'encoding': 'UTF-8',
        'info': {
            'files': [
                {'length': 4, 'path': ['root.txt']},
                {'length': 4, 'path': ['sub1', 'sub11.txt']},
                {'length': 11, 'path': ['sub1', 'sub2', 'кириллица.txt']},
                {'length': 4, 'path': ['sub1', 'sub2', 'sub22.txt']}
            ],
            'name': 'torrtest',
            'piece length': 32768,
            'pieces': b'?\x9ew\xc1A\x84\x8d\x8b\xb7\x91\x19\xe3(\x1e\x1ex\x1e\xde\xa8\xdc',
            'private': 0,
        }
    }


@pytest.fixture(scope='session')
def struct_torr_file():
    return {
        'announce': 'udp://123.123.123.123',
        'created by': 'Transmission/2.84 (14307)',
        'creation date': 1445449205,
        'encoding': 'UTF-8',
        'info': {
            'length': 4,
            'name': 'root.txt',
            'piece length': 32768,
            'pieces': b'\xa8\xfd\xc2\x05\xa9\xf1\x9c\xc1\xc7Pz`\xc4\xf0\x1b\x13\xd1\x1d\x7f\xd0',
            'private': 1,
        }
    }



import pytest

from torrentool.exceptions import RemoteUploadError, RemoteDownloadError
from torrentool.utils import get_app_version, humanize_filesize, get_open_trackers_from_local, \
    get_open_trackers_from_remote, upload_to_cache_server


def test_get_app_version():
    assert 'torrentool' in get_app_version()


def test_filesize():
    assert humanize_filesize(0) == '0 B'
    assert humanize_filesize(1024) == '1.0 KB'
    assert humanize_filesize(1024 * 1024) == '1.0 MB'


def test_get_opentrackers(monkeypatch, response_mock):

    assert isinstance(get_open_trackers_from_local(), list)

    url = 'https://raw.githubusercontent.com/idlesign/torrentool/master/torrentool/repo/open_trackers.ini'

    with response_mock(f'GET  {url} -> 200:1\n2\n'):
        assert get_open_trackers_from_remote() == ['1', '2']

    with response_mock(f'GET  {url} -> 500:'):
        with pytest.raises(RemoteDownloadError):
            get_open_trackers_from_remote()


def test_cache_upload(monkeypatch, torr_test_file, response_mock):

    url = 'http://torrage.info/autoupload.php'

    with response_mock(f'POST {url} -> 200:somehash'):
        assert upload_to_cache_server(torr_test_file) == 'http://torrage.info/torrent.php?h=somehash'

    with response_mock(f'POST {url} -> 500:'):
        with pytest.raises(RemoteUploadError):
            upload_to_cache_server(torr_test_file)



import pytest

from torrentool.api import Bencode
from torrentool.exceptions import BencodeDecodingError, BencodeEncodingError

enc = lambda v: v.encode()

decode = Bencode.read_string
encode = Bencode.encode


def read_file(filepath):
    with open(filepath, mode='rb') as f:
        contents = f.read()
    return contents


def test_read_file_dir(torr_test_dir, struct_torr_dir):
    decoded = Bencode.read_file(torr_test_dir)
    assert decoded == struct_torr_dir


def test_read_file(torr_test_file, struct_torr_file):
    decoded = Bencode.read_file(torr_test_file)
    assert decoded == struct_torr_file


def test_decode_simple():
    assert decode('4:spam') == 'spam'
    assert decode('0:') == ''

    assert decode('i3e') == 3
    assert decode('i-3e') == -3
    assert decode('i04e') == 4
    assert decode('i0e') == 0

    assert decode('l4:spam4:eggse') == ['spam', 'eggs']
    assert decode('le') == []

    assert decode('d3:cow3:moo4:spam4:eggse') == {'cow': 'moo', 'spam': 'eggs'}
    assert decode('d4:spaml1:a1:bee') == {'spam': ['a', 'b']}
    assert (
        decode('d9:publisher3:bob17:publisher-webpage15:www.example.com18:publisher.location4:homee')  ==
        {
            'publisher': 'bob',
            'publisher-webpage': 'www.example.com',
            'publisher.location': 'home',
        })
    assert decode('de') == {}


def test_decode_errors():
    with pytest.raises(BencodeDecodingError):
        Bencode.read_string('u:some')


def test_encode_simple():
    assert encode('spam') == enc('4:spam')
    assert encode('') == enc('0:')

    assert encode(b'spam') == enc('4:spam')
    assert encode(b'\xd0\xb9') == b'2:\xd0\xb9'

    assert encode(3) == enc('i3e')
    assert encode(-3) == enc('i-3e')
    assert encode(0) == enc('i0e')

    assert encode(['spam', 'eggs']) == enc('l4:spam4:eggse')
    assert encode([]) == enc('le')

    assert encode({'cow': 'moo', 'spam': 'eggs'}) == enc('d3:cow3:moo4:spam4:eggse')
    assert encode({'spam': ['a', 'b']}) == enc('d4:spaml1:a1:bee')
    assert (
        encode({
            'publisher': 'bob',
            'publisher-webpage': 'www.example.com',
            'publisher.location': 'home',
        }) ==
        enc('d9:publisher3:bob17:publisher-webpage15:www.example.com18:publisher.location4:homee')
    )
    assert encode({}) == enc('de')


def test_encode_complex(struct_torr_file, struct_torr_dir, torr_test_file, torr_test_dir):
    encoded = encode(struct_torr_file)
    from_file = read_file(torr_test_file)
    assert encoded == from_file

    encoded = encode(struct_torr_dir)
    from_file = read_file(torr_test_dir)
    assert encoded == from_file


def test_encode_errors():
    with pytest.raises(BencodeEncodingError):
        Bencode.encode(object())



import sys
import os
from setuptools import setup
from torrentool import VERSION


PATH_BASE = os.path.dirname(__file__)
PATH_BIN = os.path.join(PATH_BASE, 'bin')

PYTEST_RUNNER = ['pytest-runner'] if 'test' in sys.argv else []

f = open(os.path.join(PATH_BASE, 'README.rst'))
README = f.read()
f.close()

setup(
    name='torrentool',
    version='.'.join(map(str, VERSION)),
    url='https://github.com/idlesign/torrentool',

    description='The tool to work with torrent files.',
    long_description=README,
    license='BSD 3-Clause License',

    author='Igor `idle sign` Starikov',
    author_email='idlesign@yandex.ru',

    packages=['torrentool'],
    include_package_data=True,
    zip_safe=False,

    install_requires=[],
    setup_requires=[] + PYTEST_RUNNER,
    tests_require=[
        'pytest',
        'pytest-datafixtures',
        'pytest-responsemock',
    ],

    entry_points={
        'console_scripts': ['torrentool = torrentool.cli:main'],
    },

    test_suite='tests',

    classifiers=[
        # As in https://pypi.python.org/pypi?:action=list_classifiers
        'Development Status :: 5 - Production/Stable',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'License :: OSI Approved :: BSD License'
    ],
)
